package com.gic.haoban.manage.service.service.out.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.GlobalVar;
import com.gic.commons.util.ImageUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreGroupDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.common.utils.UuidUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.StaffDepartmentRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanStaffDepartmentRelated;
import com.gic.haoban.manage.service.entity.TabHaobanSyncErrorLog;
import com.gic.haoban.manage.service.service.*;
import com.gic.haoban.manage.service.util.EmojiFilterUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.thirdparty.api.dto.PicUploadResDTO;
import com.gic.thirdparty.api.service.QQCloudPicService;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class StaffApiServiceImpl implements StaffApiService {

    private static final Logger logger = LoggerFactory.getLogger(StaffApiServiceImpl.class);
    @Autowired
    private StaffService staffService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;
    @Autowired
    private StaffDepartmentRelatedMapper staffDepartmentRelatedMapper;

    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private com.gic.enterprise.api.service.DepartmentService gicDepartmentService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private UserLoginLogService userLoginLogService;

    @Autowired
    private SyncErrorLogService syncErrorLogService;

    @Autowired
    private QQCloudPicService qqCloudPicService;

    @Autowired
    private StaffClerkRelationService staffClerkRelationService;

    @Autowired
    private SecretSettingService secretSettingService;
    @Autowired
    private StaffPrivacyUseLogService staffPrivacyUseLogService;
    @Autowired
    private Config config;

    @Override
    public StaffDTO selectById(String staffId) {
        TabHaobanStaff staff = staffService.selectById(staffId);
        return EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
    }

    @Override
    public List<StaffDTO> listByPhoneNumber(String phoneNumber) {
        List<TabHaobanStaff> staffs = staffMapper.listByPhoneNumber(phoneNumber);
        return EntityUtil.changeEntityListByJSON(StaffDTO.class, staffs);
    }

    @Override
    public StaffDTO selectByNationcodeAndPhoneNumber(String wxEnterpriseId, String nationcode, String phoneNumber) {
        TabHaobanStaff staff = staffService.selectByNationcodeAndPhoneNumber(wxEnterpriseId, nationcode, phoneNumber);
        return EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
    }

    @Override
    public ServiceResponse add(StaffDTO staff, String departmentIds) {
        ServiceResponse hr = new ServiceResponse();
        hr.setCode(1);
        Integer activeFlag = staff.getActiveFlag();
        staff.setActiveFlag(0);
        Integer syncPostionFlag = staff.getSyncPostionFlag();
        if (syncPostionFlag != null && syncPostionFlag == 1) {
            staff.setExtendPostion(staff.getPostion());
        }
        TabHaobanStaff tab = EntityUtil.changeEntityByJSON(TabHaobanStaff.class, staff);
        String staffId = StringUtil.randomUUID();
        tab.setStaffId(staffId);
        //调接口获取
        String wxUserId = "";
        WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(staff.getWxEnterpriseId());
        if (enterpriseDTO != null) {
            UserDTO userDTO = new UserDTO();
            userDTO.setCorpid(enterpriseDTO.getCorpid());
            String[] departmentIdArr = departmentIds.split(",");
            Integer[] wxDepartmentIdArrIntegers = new Integer[departmentIdArr.length];
            int index = 0;
            for (String string : departmentIdArr) {
                TabHaobanDepartment tabs = departmentService.selectById(string);
                wxDepartmentIdArrIntegers[index] = Integer.parseInt(tabs.getWxDepartmentId());
                index++;
            }
            userDTO.setDepartment(wxDepartmentIdArrIntegers);
            userDTO.setGender(staff.getSex() == null ? null : staff.getSex().toString());
            userDTO.setMobile(staff.getPhoneNumber());
            userDTO.setName(staff.getStaffName());
            userDTO.setUserid(staffId);
            userDTO.setAlias(staff.getNickName());
            userDTO.setExternal_position(tab.getExtendPostion());
            userDTO.setPosition(tab.getPostion());
            if (activeFlag == 1) {
                userDTO.setTo_invite(true);
            }
            if (RedisUtil.getCache("is-to-invite-send") != null) {
                userDTO.setTo_invite(false);
            }
            logger.info("【店员新增】userDTO = {}", JSON.toJSONString(userDTO));
            SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.CONTACT_SECRET.getVal());
            if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
                hr.setCode(0);
                hr.setMessage("没配置通讯录secret");
                return hr;
            }
            JSONResponse jp = qywxUserApiService.createSelfWorkWxUser(userDTO, enterpriseDTO.getCorpid(), secretSetting.getSecretVal());
            logger.info("【店员新增】jp = {}", JSON.toJSONString(jp));
            if (jp.getErrorCode() != 0) {
                hr.setCode(0);
                hr.setMessage("微信新增失败:" + jp.getErrorCode());
                return hr;
            }
            staffService.add(tab);
            wxUserId = staffId;
        }
        if (StringUtils.isNotBlank(wxUserId)) {
            String[] departmentIdArr = departmentIds.split(",");
            for (String string : departmentIdArr) {
                StaffDepartmentRelatedDTO related = new StaffDepartmentRelatedDTO();
                related.setDepartmentId(string);
                related.setNationCode(staff.getNationCode());
                related.setPhoneNumber(staff.getPhoneNumber());
                related.setStaffId(staffId);
                related.setWxUserId(wxUserId);
                related.setStaffName(staff.getStaffName());
                related.setWxEnterpriseId(staff.getWxEnterpriseId());
                staffDepartmentRelatedService.add(related);
            }
        } else {
            hr.setCode(0);
            hr.setMessage("微信端新增失败");
        }
        hr.setResult(staffId);
        return hr;
    }

    @Override
    public List<StaffDepartmentRelatedDTO> listStaffDepartmentByStaffId(String staffId) {
        List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staffId);
        return EntityUtil.changeEntityListByJSON(StaffDepartmentRelatedDTO.class, list);
    }

    @Override
    public Page<StaffDTO> pageStaff(List<String> departmentIds, Integer activeFlag, String keyword, BasePageInfo pageInfo, Integer relationFlag) {
        // 部门 员工列表
        List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listByDepartmentIdByKeyword(departmentIds, keyword);

        if (CollectionUtils.isEmpty(list)) {
            return new Page<>();
        }

        // 当前部门的所有员工id列表
        Set<String> staffIdSet = list.stream().map(TabHaobanStaffDepartmentRelated::getStaffId).collect(Collectors.toSet());

        // 查询当前部门所有已关联导购的员工id列表
        List<String> relationsStaffIdList = staffClerkRelationService.listRelationsStaffId(staffIdSet);

        Set<String> relationStaffSet = new HashSet<>(relationsStaffIdList);

        logger.info("已关联的员工id列表:{}", JSON.toJSONString(relationStaffSet));

        Set<String> queryStaffSet = filterByRelationFlag(staffIdSet, relationStaffSet, relationFlag);

        if (CollectionUtils.isEmpty(queryStaffSet)) {
            return new Page<>();
        }

        // 分页
        PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
        Page<StaffDTO> page = PageUtil.changePageHelperToCurrentPage(staffService.pageStaff(queryStaffSet, activeFlag, ""), StaffDTO.class);

        if (CollectionUtils.isEmpty(page.getResult())) {
            return new Page<>();
        }

        Map<String, TabHaobanStaffDepartmentRelated> map = list.stream()
                .collect(Collectors.toMap(TabHaobanStaffDepartmentRelated::getStaffId, Function.identity(), (s1, s2) -> s2));

        page.getResult().forEach(one -> {
            TabHaobanStaffDepartmentRelated related = map.get(one.getStaffId());
            one.setDepartmentIds(Objects.isNull(related) ? null : related.getDepartmentId());
            // 关联状态
            one.setRelationFlag(relationStaffSet.contains(one.getStaffId()) ? 1 : 0);
        });

        return page;
    }

    private Set<String> filterByRelationFlag(Set<String> sourceStaffSet, Set<String> relationStaffSet, Integer relationFlag) {
        // 为空默认全部
        if (Objects.isNull(relationFlag)) {
            return sourceStaffSet;
        }
        // 已关联 1 未关联 0
        if (relationFlag == 1) {
            // 已关联用已关联id查询
            return relationStaffSet;
        }
        // 未关联 0 , 取原集合与关联集合的-差集
        sourceStaffSet.removeAll(relationStaffSet);
        // 差集处理成功返回
        return sourceStaffSet;
    }

    @Override
    public List<StaffDTO> listByIds(List<String> staffIds) {
        if (CollectionUtil.isEmpty(staffIds)) {
            return new ArrayList<StaffDTO>();
        }
        List<TabHaobanStaff> staffs = staffMapper.listByIds(staffIds);
        if (CollectionUtil.isNotEmpty(staffs)) {
            List<StaffDTO> resultList = EntityUtil.changeEntityListByJSON(StaffDTO.class, staffs);
            return resultList;
        } else {
            return new ArrayList<StaffDTO>();
        }
    }

    @Override
    public void del(String staffDepartmentStaffRelatedId) {
        StaffDepartmentRelatedDTO related = staffDepartmentRelatedService.getById(staffDepartmentStaffRelatedId);
        if (related != null) {
            List<TabHaobanStaffDepartmentRelated> relatedList = staffDepartmentRelatedService.listStaffDepartmentByStaffId(related.getStaffId());
            if (relatedList == null || relatedList.isEmpty()) {
                return;
            }
            //只在当前门店
            if (relatedList.size() == 1) {
                TabHaobanStaff staff = staffService.selectById(relatedList.get(0).getStaffId());
                WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(staff.getWxEnterpriseId());
                if (enterpriseDTO != null) {
                    ServiceResponse delQywxStaff = delQywxStaff(staff, enterpriseDTO);
//					JSONResponse jp =  qywxUserApiService.deleteSelfWorkWxUser(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), staff.getWxUserId());
                    logger.info("【删除成员】{} ,jp={}", staffDepartmentStaffRelatedId, JSON.toJSONString(delQywxStaff));
                    if (delQywxStaff.getCode() != 1) {
                        throw new RuntimeException("企业微信删除失败");
                    }
                }
                staffDepartmentRelatedService.del(related);
//				staffService.delByuserid(relatedList.get(0).getStaffId());

                TabHaobanStaffDepartmentRelated staffRelated = relatedList.get(0);
                if (StringUtils.isNotBlank(staffRelated.getClerkCode())) {
                    cleanGicClerk(staffRelated.getClerkCode(), related.getDepartmentId());
                }
                //走修改流程
            } else {
                String departmentIds = "";
                for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : relatedList) {
                    if (!staffDepartmentStaffRelatedId.equals(tabHaobanStaffDepartmentRelated.getStaffDepartmentRelatedId())) {
                        departmentIds += tabHaobanStaffDepartmentRelated.getDepartmentId() + ",";
                    }
                }
                if (StringUtils.isNotBlank(departmentIds)) {
                    departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                }
                TabHaobanStaff staff = staffService.selectById(relatedList.get(0).getStaffId());
                staffEdit(EntityUtil.changeEntityByJSON(StaffDTO.class, staff), departmentIds);
            }
        }
    }

    @Override
    public void cleanGicClerk(String code, String departmentId) {
        if (true) {
            return;
        }
        if (StringUtils.isNotBlank(code)) {
            TabHaobanDepartment department = departmentService.selectById(departmentId);
            String enterpriseId = "";
            if (department.getIsStore() == 0) {
                com.gic.enterprise.api.dto.DepartmentDTO dto = gicDepartmentService.getDeptment(department.getRelatedId());
                if (dto == null) {
                    StoreGroupDTO group = storeGroupService.getStoreGroupById(department.getRelatedId());
                    if (group != null) {
                        enterpriseId = group.getEnterpriseId();
                    }
                } else {
                    enterpriseId = dto.getEnterpriseId();
                }
            } else {
                StoreDTO store = storeService.getStore(department.getRelatedId());
                enterpriseId = store.getEnterpriseId();
            }
            ClerkDTO clerkDTO = clerkService.getClerkByCodeNoStatus(enterpriseId, code);
            if (clerkDTO != null) {
                if (!clerkDTO.getPhoneNumber().equals("--") && StringUtils.isNotBlank(clerkDTO.getPhoneNumber())) {
                    logger.info("【员工修改】clerkDTO={}", JSON.toJSONString(clerkDTO));
                    clerkDTO.setPhoneNumber("--");
                    clerkService.updateClerk(clerkDTO);
                }
            }
        }
    }

    @Override
    public ServiceResponse getWxSaveNew(String userId, String wxEnterpriseId) {
        ServiceResponse res = new ServiceResponse();
        WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (null == enterpriseDTO) {
            logger.info("企业不存在:{}:{}", wxEnterpriseId, userId);
            res.setMessage("企业不存在");
            res.setCode(2);
            return res;
        }

        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
            logger.info("没有配置secret：{}", JSONObject.toJSONString(enterpriseDTO));
            res.setMessage("没有配置secret");
            res.setCode(4);
            return res;
        }
        UserDTO user = qywxUserApiService.getSelfWorkWxUser(enterpriseDTO.getCorpid(), secretSetting.getSecretVal(), userId);
        if (null == user) {
            logger.info("企业微信用户不存在:{}:{}", wxEnterpriseId, userId);
            res.setMessage("企业微信用户不存在");
            res.setCode(3);
            return res;
        }
        String imageUrl = changeHeaderImageUrl(user.getAvatar());
        String mobile = user.getMobile();
        String[] arr = getNationCodeAndPhoneNumber(mobile);
        String nationCode = arr[0];
        String phoneNumber = arr[1];
        TabHaobanStaff staff = staffService.selectByPhoneNumberAndEnterpriseId(phoneNumber, wxEnterpriseId);
        if (staff != null) {
            staffService.delOtherStaffByWxUserId(staff.getWxUserId(), staff.getStaffId(), wxEnterpriseId);
        }
        staff = (staff == null ? new TabHaobanStaff() : staff);
        staff.setWxUserId(userId);
        staff.setUpdateTime(new Date());
        staff.setStatusFlag(1);
        staff.setWxEnterpriseId(wxEnterpriseId);
        if (StringUtils.isNotBlank(user.getAvatar())) {
            staff.setHeadImg(imageUrl);
        }
        if (StringUtils.isNotBlank(user.getName())) {
            user.setName(EmojiFilterUtil.removeAllEmoji(user.getName()));
            staff.setStaffName((user.getName()));
        }
        if (StringUtils.isNotBlank(user.getAlias())) {
            user.setAlias(EmojiFilterUtil.removeAllEmoji(user.getAlias()));
            staff.setNickName((user.getAlias()));
        }
        staff.setPhoneNumber(phoneNumber);
        staff.setNationCode(nationCode);
        staff.setSex(user.getGender() == null ? 1 : Integer.parseInt(user.getGender()));
        staff.setPostion(user.getPosition());
        staff.setExtendPostion(user.getExternal_position());
        String qr_code = user.getQr_code();
        if (StringUtils.isNotBlank(qr_code)) {
            staff.setQrCode(qr_code);
        }
        //成员更改或更新
        if (StringUtils.isBlank(staff.getStaffId())) {
            logger.info("新增成员");
            staff.setActiveFlag(0);
            staff.setCreateTime(new Date());
            staffService.add(staff);
        } else {
            staffService.updateByPrimaryKey(staff);
        }
        logger.info("门店变更保存：{}", JSONObject.toJSONString(user));
        staffDepartChange(staff, wxEnterpriseId, user);
        return res;
    }

    private String[] getNationCodeAndPhoneNumber(String phoneNumber) {
        String[] arr = new String[2];
        if (StringUtils.isBlank(phoneNumber)) {
            arr[0] = "";
            arr[1] = "";
            return arr;
        }
        arr[0] = "86";
        arr[1] = phoneNumber;
        if (phoneNumber.startsWith("+853")) {
            arr[0] = "853";
            arr[1] = phoneNumber.replace("+853", "");
        }
        if (phoneNumber.startsWith("+852")) {
            arr[0] = "852";
            arr[1] = phoneNumber.replace("+852", "");
        }
        if (phoneNumber.startsWith("+886")) {
            arr[0] = "886";
            arr[1] = phoneNumber.replace("+886", "");
        }
        if (phoneNumber.startsWith("+65")) {
            arr[0] = "65";
            arr[1] = phoneNumber.replace("+65", "");
        }
        if (phoneNumber.startsWith("+66")) {
            arr[0] = "66";
            arr[1] = phoneNumber.replace("+66", "");
        }
        return arr;
    }

    private void staffDepartChange(TabHaobanStaff staff, String wxEnterpriseId, UserDTO user) {

        //更新部门
        Integer[] departmentIds = user.getDepartment();
        List<String> wxDepartIds = new ArrayList<>();
        for (Integer wDId : departmentIds) {
            wxDepartIds.add(wDId.toString());
        }

        //关联部门
        List<DepartmentDTO> departmentDTOList = departmentService.listDepartmentByWxDepartIds(wxEnterpriseId, wxDepartIds);
        if (CollectionUtils.isEmpty(departmentDTOList)) {
            logger.info("部门没有更新或同步：{}", wxEnterpriseId);
            return;
        }

        //已存在部门
        List<TabHaobanStaffDepartmentRelated> hasRelation = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
        Map<String, TabHaobanStaffDepartmentRelated> hasRelationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(hasRelation)) {
            hasRelationMap = hasRelation.stream().collect(Collectors.toMap(TabHaobanStaffDepartmentRelated::getDepartmentId, tab -> tab));
        }
        Map<String, TabHaobanStaffDepartmentRelated> finalHasRelationMap = hasRelationMap;
        TabHaobanStaff finalStaff = staff;
        //新增操作
        departmentDTOList.forEach(departmentDTO -> {
            if (!finalHasRelationMap.containsKey(departmentDTO.getDepartmentId())) {
                StaffDepartmentRelatedDTO dto = new StaffDepartmentRelatedDTO();
                dto.setPhoneNumber(staff.getPhoneNumber());
                dto.setWxUserId(user.getUserid());
                dto.setStaffId(finalStaff.getStaffId());
                dto.setStaffName(user.getName());
                dto.setWxEnterpriseId(wxEnterpriseId);
                dto.setStatusFlag(1);
                dto.setDepartmentId(departmentDTO.getDepartmentId());
                dto.setNationCode(staff.getNationCode());
                dto.setStaffName(user.getName());
                staffDepartmentRelatedService.add(dto);
            }
        });
        //删除操作
        Set<String> newDepartIds = departmentDTOList.stream().map(DepartmentDTO::getDepartmentId).collect(Collectors.toSet());
        Set<String> hasDepartIds = hasRelationMap.keySet();
        Sets.SetView<String> delDepartIds = Sets.difference(hasDepartIds, newDepartIds);
        Sets.SetView<String> updateDepartIds = Sets.intersection(newDepartIds, hasDepartIds);//交集
        logger.info("删除部门：{}", JSONObject.toJSONString(delDepartIds));
        if (!delDepartIds.isEmpty()) {
            delDepartIds.forEach(delDepartId -> {
                TabHaobanStaffDepartmentRelated departmentRelated = finalHasRelationMap.get(delDepartId);
                if (null != departmentRelated) {
                    StaffDepartmentRelatedDTO relatedDTO = new StaffDepartmentRelatedDTO();
                    relatedDTO.setStaffDepartmentRelatedId(departmentRelated.getStaffDepartmentRelatedId());
                    staffDepartmentRelatedService.del(relatedDTO);
                }
            });
        }

        if (!updateDepartIds.isEmpty()) {
            updateDepartIds.forEach(updateDeId -> {
                TabHaobanStaffDepartmentRelated departmentRelated = finalHasRelationMap.get(updateDeId);
                if (null != departmentRelated) {
                    departmentRelated.setPhoneNumber(staff.getPhoneNumber());
                    departmentRelated.setStaffName(staff.getStaffName());
                    departmentRelated.setWxUserId(staff.getWxUserId());
                    departmentRelated.setNationCode(staff.getNationCode());
                    departmentRelated.setUpdateTime(new Date());
                    staffDepartmentRelatedService.update(departmentRelated);
                }
            });
        }
    }

    @Override
    public void wxGetAdd(String userId, String wxEnterpriseId) {
        RedisUtil.lock("haoban_add_user" + wxEnterpriseId + userId, 2L);
        ServiceResponse wxSaveNew = getWxSaveNew(userId, wxEnterpriseId);
        logger.info("信息:{}", JSONObject.toJSONString(wxSaveNew));
    }

    @Deprecated
    public void wxGetAddOld(String userId, String wxEnterpriseId) {
        //获取微信用户信息

        TabHaobanStaff staff = staffService.selectByUserIdAndEnterpriseId(userId, wxEnterpriseId);
        if (staff == null) {
            WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
            if (enterpriseDTO != null) {
                Date now = new Date();
                SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.HAOBAN_HELP.getVal());
                if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
                    logger.info("没有配置secret");
                    return;
                }
                UserDTO user = qywxUserApiService.getSelfWorkWxUser(enterpriseDTO.getCorpid(), secretSetting.getSecretVal(), userId);
                if (user == null) {
                    return;
                }
                logger.info("【新增用户】user={}", JSON.toJSONString(user));
                TabHaobanStaff tab = new TabHaobanStaff();
                tab.setWxUserId(userId);
                tab.setCreateTime(now);
                tab.setUpdateTime(now);
                tab.setStatusFlag(1);
                tab.setWxEnterpriseId(wxEnterpriseId);
                if (StringUtils.isNotBlank(user.getAvatar())) {
                    tab.setHeadImg(changeHeaderImageUrl(user.getAvatar()));
                }
                tab.setPhoneNumber(user.getMobile());
                if (StringUtils.isNotBlank(user.getName())) {
                    user.setName(EmojiFilterUtil.removeAllEmoji(user.getName()));
                    tab.setStaffName((user.getName()));
                }
                if (StringUtils.isNotBlank(user.getAlias())) {
                    user.setAlias(EmojiFilterUtil.removeAllEmoji(user.getAlias()));
                    tab.setNickName((user.getAlias()));
                }
//					tab.setNickName(user.getAlias());
                tab.setNationCode("86");
                tab.setSex(user.getGender() == null ? 1 : Integer.parseInt(user.getGender()));
                tab.setActiveFlag(1);
                tab.setPostion(user.getPosition());
                tab.setExtendPostion(user.getExternal_position());
                if (user.getStatus() != null && user.getStatus() == 1) {
                    tab.setActiveFlag(1);
                } else {
                    tab.setActiveFlag(0);
                }
                String staffId = staffService.add(tab);

                Integer[] departmentId = user.getDepartment();
                for (Integer integer : departmentId) {
                    TabHaobanDepartment tabDepartment = departmentService.getByWxId(integer.toString(), wxEnterpriseId);
                    if (tabDepartment != null) {
                        TabHaobanStaffDepartmentRelated tabStaff = staffDepartmentRelatedService.getDepartmentIdAndStaffId(tabDepartment.getDepartmentId(), tab.getStaffId());
                        if (tabStaff == null) {
                            StaffDepartmentRelatedDTO dto = new StaffDepartmentRelatedDTO();
                            dto.setPhoneNumber(user.getMobile());
                            dto.setWxUserId(userId);
                            dto.setStaffId(staffId);
                            dto.setStaffName(user.getName());
                            dto.setWxEnterpriseId(wxEnterpriseId);
                            dto.setStatusFlag(1);
                            dto.setDepartmentId(tabDepartment.getDepartmentId());
                            dto.setNationCode("86");
                            dto.setStaffName(user.getName());
                            staffDepartmentRelatedService.add(dto);
                        }
                    }
                }
            }
        } else {
            WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
            if (enterpriseDTO != null) {
                SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.CONTACT_SECRET.getVal());
                if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
                    logger.info("没有设置通讯录secret");
                    return;
                }
                UserDTO user = qywxUserApiService.getSelfWorkWxUser(enterpriseDTO.getCorpid(), secretSetting.getSecretVal(), userId);
                logger.info("【修改用户】user={}", JSON.toJSONString(user));
                Integer[] departmentId = user.getDepartment();
                String departmentIds = "";
                for (Integer integer : departmentId) {
                    TabHaobanDepartment tabDepartment = departmentService.getByWxId(integer.toString(), wxEnterpriseId);
                    if (tabDepartment != null) {
                        departmentIds += tabDepartment.getDepartmentId() + ",";
                    }
                }
                if (StringUtils.isNotBlank(departmentIds)) {
                    departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                }
                StaffDTO staffDTO = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
                staffDTO.setWeixinPush(true);
                if (StringUtils.isBlank(staffDTO.getPhoneNumber())) {
                    staffDTO.setPhoneNumber(user.getMobile());
                    staffDTO.setSex(StringUtils.isBlank(user.getGender()) ? null : Integer.parseInt(user.getGender()));
                    staffDTO.setHeadImg(user.getAvatar());
                    staffDTO.setPostion(user.getPosition());
                    staffDTO.setNickName(user.getAlias());
                    staffDTO.setExtendPostion(user.getExternal_position());
                    Integer status = user.getStatus();
                    if (status != null && status == 2) {
                        staffDTO.setActiveFlag(1);
                    }
                }

                this.staffEdit(staffDTO, departmentIds);
                //String staffId = staffService.add(tab);
            }
        }
    }


    @Override
    public void wxFristAdd(String userJson, String wxEnterpriseId) {
        if (StringUtils.isBlank(userJson)) {
            return;
        }
        UserDTO user = JSON.parseObject(userJson, UserDTO.class);
        //获取微信用户信息
        String userId = user.getUserid();
        TabHaobanStaff staff = staffService.selectByUserIdAndEnterpriseId(userId, wxEnterpriseId);
        if (staff == null) {
            WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
            if (enterpriseDTO != null) {
                Date now = new Date();
                //UserDTO user = qywxUserApiService.getSelfWorkWxUser(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), userId);
                logger.info("【新增用户】user={}", JSON.toJSONString(user));
                TabHaobanStaff tab = new TabHaobanStaff();
                tab.setWxUserId(userId);
                tab.setCreateTime(now);
                tab.setUpdateTime(now);
                tab.setStatusFlag(1);
                tab.setWxEnterpriseId(wxEnterpriseId);
                if (StringUtils.isNotBlank(user.getAvatar())) {
                    tab.setHeadImg(changeHeaderImageUrl(user.getAvatar()));
                }
                tab.setPhoneNumber(user.getMobile());
                if (StringUtils.isNotBlank(user.getName())) {
                    user.setName(EmojiFilterUtil.removeAllEmoji(user.getName()));
                    tab.setStaffName((user.getName()));
                }
                if (StringUtils.isNotBlank(user.getAlias())) {
                    user.setAlias(EmojiFilterUtil.removeAllEmoji(user.getAlias()));
                    tab.setNickName((user.getAlias()));
                }
//					tab.setNickName(user.getAlias());
                tab.setNationCode("86");
                tab.setSex(user.getGender() == null ? 1 : Integer.parseInt(user.getGender()));
                tab.setActiveFlag(1);
                tab.setPostion(user.getPosition());
                tab.setExtendPostion(user.getExternal_position());
                if (user.getStatus() != null && user.getStatus() == 1) {
                    tab.setActiveFlag(1);
                } else {
                    tab.setActiveFlag(0);
                }
                String staffId = staffService.add(tab);
            }
        } else {
            WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
            if (enterpriseDTO != null) {
                logger.info("【修改用户】user={}", JSON.toJSONString(user));
                Integer[] departmentId = user.getDepartment();
                String departmentIds = "";
                for (Integer integer : departmentId) {
                    TabHaobanDepartment tabDepartment = departmentService.getByWxId(integer.toString(), wxEnterpriseId);
                    if (tabDepartment != null) {
                        departmentIds += tabDepartment.getDepartmentId() + ",";
                    }
                }
                if (StringUtils.isNotBlank(departmentIds)) {
                    departmentIds = departmentIds.substring(0, departmentIds.length() - 1);
                }
                StaffDTO staffDTO = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
                staffDTO.setWeixinPush(true);
                this.staffEdit(staffDTO, departmentIds);
            }
        }
    }

    private void staffEditDeal(StaffDTO staffDTO, String departmentIds, boolean syncGic) {
        TabHaobanStaff oldStaff = staffMapper.selectByPrimaryKey(staffDTO.getStaffId());
        logger.info("【员工修改】oldStaff={}", JSON.toJSONString(oldStaff));
        //1、先更新staff
        logger.info("【员工修改】staffDTO={}", JSON.toJSONString(staffDTO));
        TabHaobanStaff tab = EntityUtil.changeEntityByJSON(TabHaobanStaff.class, staffDTO);
        staffService.updateByPrimaryKey(tab);
        String staffName = tab.getStaffName();

        List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staffDTO.getStaffId());
        Map<String, TabHaobanStaffDepartmentRelated> map = com.gic.commons.util.CollectionUtil.toMap(list, "departmentId");
        //2、该员工新增部门
        if (StringUtils.isNotBlank(departmentIds)) {
            String[] addIds = departmentIds.split(",");
            for (String addId : addIds) {
                if (!map.containsKey(addId)) {
                    //该员工新增了部门
                    StaffDepartmentRelatedDTO related = new StaffDepartmentRelatedDTO();
                    related.setStaffDepartmentRelatedId(UuidUtil.randomUUID());
                    related.setStaffId(staffDTO.getStaffId());
                    related.setClerkCode(null);
                    related.setCreateTime(new Date());
                    related.setDepartmentId(addId);
                    related.setNationCode(staffDTO.getNationCode());
                    related.setPhoneNumber(staffDTO.getPhoneNumber());
                    related.setStatusFlag(1);
                    related.setUpdateTime(new Date());
                    related.setWxUserId(oldStaff.getWxUserId());
                    related.setWxEnterpriseId(staffDTO.getWxEnterpriseId());
                    related.setStaffName(staffName);
                    staffDepartmentRelatedService.add(related);
                } else {
                    //该员工部门没做改变(只更新手机号即可)
                    TabHaobanStaffDepartmentRelated related = map.get(addId);
                    if (!staffDTO.getPhoneNumber().equals(related.getPhoneNumber())) {
                        related.setPhoneNumber(staffDTO.getPhoneNumber());
                        related.setNationCode(staffDTO.getNationCode());
                        related.setUpdateTime(new Date());
                        related.setStaffName(staffName);
                        staffDepartmentRelatedMapper.updateByPrimaryKeySelective(related);
                    }
                }
            }
        }
        //3、该员工删除部门
        for (TabHaobanStaffDepartmentRelated tab1 : list) {
            if (departmentIds == null) {
                departmentIds = "";
            }
            //不包含，则说明员工删除了该部门
            if (!departmentIds.contains(tab1.getDepartmentId())) {
                logger.info("【员工修改删除】tab1.getDepartmentId()={}", tab1.getDepartmentId());
                StaffDepartmentRelatedDTO related = EntityUtil.changeEntityByJSON(StaffDepartmentRelatedDTO.class, tab1);
                staffDepartmentRelatedService.del(related);
            }
        }

        Boolean weixinPush = staffDTO.getWeixinPush();
        if (weixinPush == null || !weixinPush) {
            updateWxMessage(oldStaff, staffDTO, departmentIds);
        }
    }

    private void wxStaffEdit(StaffDTO staffDTO, String departmentIds) {
        staffDTO.setWeixinPush(true);
        staffEditDeal(staffDTO, departmentIds, false);
    }


    private void updateGicClerk(TabHaobanStaff oldStaff, StaffDTO staffDTO, String departmentId, TabHaobanStaffDepartmentRelated related) {
        logger.info("【员工修改】oldName={},name={},oldPhoneNumber={},phoneNumber={}", oldStaff.getStaffName(), staffDTO.getStaffName(), oldStaff.getPhoneNumber(), staffDTO.getPhoneNumber());
        String staffName = staffDTO.getStaffName();
        TabHaobanDepartment department = departmentService.selectById(departmentId);
        if (department == null) {
            return;
        }
        String enterpriseId = "";
        if (department.getIsStore() == null || department.getIsStore() == 0) {
            com.gic.enterprise.api.dto.DepartmentDTO dto = gicDepartmentService.getDeptment(department.getRelatedId());
            if (dto == null) {
                StoreGroupDTO group = storeGroupService.getStoreGroupById(department.getRelatedId());
                if (group != null) {
                    enterpriseId = group.getEnterpriseId();
                }
            } else {
                enterpriseId = dto.getEnterpriseId();
            }
        } else {
            StoreDTO store = storeService.getStore(department.getRelatedId());
            if (store == null) {
                return;
            }
            enterpriseId = store.getEnterpriseId();
        }
        ClerkDTO clerkDTO = clerkService.getClerkByCodeNoStatus(enterpriseId, related.getClerkCode());
        logger.info("clerkDto:{}", JSONObject.toJSONString(clerkDTO));
        if (null == clerkDTO) {
            return;
        }
        String headPic = staffDTO.getHeadImg();
        if (StringUtils.isBlank(staffDTO.getPhoneNumber())) {
            staffDTO.setPhoneNumber("");
        }
        if (StringUtils.isBlank(clerkDTO.getPhoneNumber())) {
            clerkDTO.setPhoneNumber("");
        }
        if (clerkDTO != null && (StringUtils.isNotBlank(staffName) && !clerkDTO.getClerkName().equals(staffName)
                || !staffDTO.getPhoneNumber().equals(clerkDTO.getPhoneNumber()))
                || (org.apache.commons.lang3.StringUtils.isNotBlank(headPic) && !headPic.equals(clerkDTO.getHeadImgUrl()))
                || !staffDTO.getSex().equals(Integer.valueOf(clerkDTO.getClerkGender()))) {
            logger.info("【员工修改】clerkDTO={}", JSON.toJSONString(clerkDTO));
            logger.info("【员工修改】{},修改地方：name:{},{};num:{},{};pic:{},{};sex:{},{}", clerkDTO.getClerkId(), staffName
                    , clerkDTO.getClerkName(), staffDTO.getPhoneNumber(), clerkDTO.getPhoneNumber(), headPic, clerkDTO.getHeadImgUrl()
                    , staffDTO.getSex(), clerkDTO.getClerkGender());
            clerkDTO.setClerkName(staffDTO.getStaffName());
            clerkDTO.setPhoneNumber(staffDTO.getPhoneNumber());
            clerkDTO.setHeadImgUrl(staffDTO.getHeadImg());
            clerkDTO.setClerkGender(staffDTO.getSex().toString());
            clerkService.updateClerk(clerkDTO);
        }
    }


    private void updateWxMessage(TabHaobanStaff oldStaff, StaffDTO staffDTO, String departmentIds) {
        WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(oldStaff.getWxEnterpriseId());
        if (enterpriseDTO != null) {
            UserDTO userDTO = new UserDTO();
            userDTO.setCorpid(enterpriseDTO.getCorpid());
            userDTO.setAlias(staffDTO.getNickName());

            String[] departmentIdArr = departmentIds.split(",");
            Integer[] wxDepartmentIdArrIntegers = new Integer[departmentIdArr.length];
            int index = 0;
            for (String string : departmentIdArr) {
                TabHaobanDepartment tabs = departmentService.selectById(string);
                wxDepartmentIdArrIntegers[index] = Integer.parseInt(tabs.getWxDepartmentId());
                index++;
            }
            userDTO.setDepartment(wxDepartmentIdArrIntegers);
            userDTO.setGender(staffDTO.getSex() == null ? "" : staffDTO.getSex().toString());
            userDTO.setMobile(staffDTO.getPhoneNumber());
            userDTO.setName(staffDTO.getStaffName());
            userDTO.setUserid(oldStaff.getWxUserId());
            userDTO.setPosition(staffDTO.getPostion());
            userDTO.setExternal_position(staffDTO.getExtendPostion());
            SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.CONTACT_SECRET.getVal());
            if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
                throw new RuntimeException("没有设置通讯录secret");
            }
            JSONResponse jp = qywxUserApiService.updateSelfWorkWxUser(userDTO, enterpriseDTO.getCorpid(), secretSetting.getSecretVal());
            if (jp.getErrorCode() != 0) {
                throw new RuntimeException("微信新增失败");
            }
        }
    }

    @Override
    public void staffEdit(StaffDTO staffDTO, String departmentIds) {
        staffEditDeal(staffDTO, departmentIds, false);
    }


    @Override
    public StaffDTO selectByUserIdAndEnterpriseId(String userId, String wxEnterpriseId) {
        return EntityUtil.changeEntityByJSON(StaffDTO.class, staffService.selectByUserIdAndEnterpriseId(userId, wxEnterpriseId));
    }

    @Override
    public StaffDepartmentRelatedDTO getDepartmentIdAndStaffId(String departmentId, String staffId) {
        TabHaobanStaffDepartmentRelated tab = staffDepartmentRelatedService.getDepartmentIdAndStaffId(departmentId, staffId);
        return EntityUtil.changeEntityByJSON(StaffDepartmentRelatedDTO.class, tab);
    }

    @Override
    public int countByDepartmentId(String departmentId) {

        return staffDepartmentRelatedService.countByDepartmentId(departmentId);
    }

    @Override
    public StaffDTO selectSuperByWxEnterpriseId(String wxEnterpriseId) {
        TabHaobanStaff tab = staffMapper.selectSuperByWxEnterpriseId(wxEnterpriseId);
        return EntityUtil.changeEntityByJSON(StaffDTO.class, tab);
    }

    @Override
    public void addGicClerk(GicClerkDTO clerkDTO) {
        logger.info("同步gic店员：{}", JSONObject.toJSONString(clerkDTO));
        addClerkByGic(clerkDTO);

    }


    @Override
    public void syncGicClerk(String fieldListString) {
        List<SyncJsonColumnDTO> fieldList = JSONArray.parseArray(fieldListString, SyncJsonColumnDTO.class);
        if (true) {
            return;
        }
    }


    private void addClerkByGic(GicClerkDTO clerkDTO) {

        String gicStoreId = clerkDTO.getStoreId();
        String code = clerkDTO.getClerkCode();
        String staffName = clerkDTO.getClerkName();
        String phoneNumber = clerkDTO.getPhoneNumber();
        String nationcode = clerkDTO.getNationcode();
        Integer sex = clerkDTO.getClerkGender();
        String headPic = clerkDTO.getHeadImgUrl();
        Integer status = clerkDTO.getStatus();
        if (status == null) {
            status = 1;
        }
        String postion = clerkDTO.getPostionName();
        String enterpriseId = clerkDTO.getEnterpriseId();

        if (StringUtils.isBlank(gicStoreId)) {
            logger.info("【店员同步】gicStoreId为空");
            return;
        }
        TabHaobanDepartment department = departmentService.selectByRelatedId(gicStoreId);
        if (department == null) {
            logger.info("【店员同步】失败====department为空");
            return;
        }
        String departmentId = department.getDepartmentId();
        String wxEnterpriseId = department.getWxEnterpriseId();
//	    StaffDTO staff = this.selectByNationcodeAndPhoneNumber(wxEnterpriseId, nationcode, phoneNumber);
        StaffDTO staff = geCcheckAndInitUser(wxEnterpriseId, nationcode, phoneNumber);
        if (staff == null) {
            if (status != 1) {
                return;
            } else {
                staff = new StaffDTO();
                staff.setWxEnterpriseId(wxEnterpriseId);
                staff.setClerkCode(code);
                staff.setActiveFlag(1);
                staff.setDepartmentIds(departmentId);
                staff.setHeadImg(headPic);
                staff.setNationCode(nationcode);
                staff.setPhoneNumber(phoneNumber);
                staff.setPostion(postion);
                staff.setSex(sex);
                staff.setStaffName(staffName);
                staff.setNickName(staffName);
                staff.setSyncPostionFlag(1);
                StaffDepartmentRelatedDTO isCodeExist = this.getDepartmentIdAndCode(departmentId, code);
                logger.info("【gic导购新增】isCodeExist={}", JSON.toJSONString(isCodeExist));
                //如果这个code已经绑定了 先解绑
                if (isCodeExist != null) {
                    isCodeExist.setClerkCode(null);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, isCodeExist));
                }
                ServiceResponse hr = this.add(staff, department.getDepartmentId());
                if (hr.getCode() == 1) {
                    StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, hr.getResult().toString());
                    logger.info("【gic导购新增】staffRelated={}", JSON.toJSONString(staffRelated));
                    //绑定
                    if (staffRelated != null) {
                        staffRelated.setClerkCode(code);
                        staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                    }
                }
            }
        } else {
            String staffId = staff.getStaffId();
            logger.info("【员工修改】staff={},status={}", JSON.toJSONString(staff), status);
            //删除
            if (status == 0) {
                StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, staffId);
                if (staffRelated != null) {
                    staffRelated.setUpdateTime(new Date());
                    staffRelated.setClerkCode(null);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                }
                //修改
            } else {
                staff.setHeadImg(headPic);
                staff.setSex(sex);
                //staff.setPostion(postion);
//	    		staff.setStaffName(staffName);
                if (staff.getActiveFlag() != null && staff.getActiveFlag() != 1) {
                    staff.setStaffName(staffName);
                }
                List<StaffDepartmentRelatedDTO> relatedList = this.listStaffDepartmentByStaffId(staffId);
                Map<String, StaffDepartmentRelatedDTO> map = com.gic.commons.util.CollectionUtil.toMap(relatedList, "departmentId");
                String departmentIds = "";
                if (map.get(departmentId) == null) {
                    departmentIds = departmentId;
                }
                for (StaffDepartmentRelatedDTO staffDepartmentRelatedDTO : relatedList) {
                    departmentIds += "," + staffDepartmentRelatedDTO.getDepartmentId();
                }
                if (departmentIds.startsWith(",")) {
                    departmentIds = departmentIds.substring(1);
                }
                if (map.get(departmentId) == null) {
                    this.staffEditDeal(staff, departmentIds, true);
                }
                StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, staffId);
                StaffDepartmentRelatedDTO isCodeExist = this.getDepartmentIdAndCode(departmentId, code);
                //如果这个人已经绑定了  先解绑
                if (isCodeExist != null) {
                    staffRelated.setUpdateTime(new Date());
                    isCodeExist.setClerkCode(null);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, isCodeExist));
                }
                //如果这个人是未绑定的 则绑定clerk
                if (staffRelated != null) {
                    staffRelated.setUpdateTime(new Date());
                    staffRelated.setClerkCode(code);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                }
                this.staffEditDeal(staff, departmentIds, true);
            }
        }
    }

    private StaffDTO geCcheckAndInitUser(String wxEnterpriseId, String nationcode, String phoneNumber) {
        logger.info("校验获取信息：{}", phoneNumber);
        StaffDTO staff = this.selectByNationcodeAndPhoneNumber(wxEnterpriseId, nationcode, phoneNumber);
        if (null != staff) {
            return staff;
        }
        WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);

        if (null == enterpriseDTO) {
            return null;
        }
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
            logger.info("没有配置secret:{}", wxEnterpriseId);
            return null;
        }
        String userId = qywxUserApiService.getSelfWxUserIdByPhonenumber(enterpriseDTO.getCorpid(), secretSetting.getSecretVal(), phoneNumber);
        logger.info("获取userId:{}，{}", phoneNumber, userId);
        if (userId == null) {
            return null;
        }
        wxGetAdd(userId, wxEnterpriseId);
        return this.selectByUserIdAndEnterpriseId(userId, wxEnterpriseId);
    }

    @Override
    public StaffDepartmentRelatedDTO getDepartmentIdAndCode(String departmentId, String code) {
        return EntityUtil.changeEntityByJSON(StaffDepartmentRelatedDTO.class, staffDepartmentRelatedMapper.getDepartmentIdAndCode(departmentId, code));
    }

    @Override
    public void initWxUser(String corpid, String contactSecret, String wxEnterpriseId, Integer wxDepartmentId, String taskId) {
        List<UserDTO> list = this.qywxUserApiService.listSelfDepartmentUser(corpid, contactSecret, wxDepartmentId.toString(), 0);
        if (CollectionUtils.isNotEmpty(list)) {
            for (UserDTO dto : list) {
                try {
                    this.wxGetAdd(dto.getUserid(), wxEnterpriseId);
                } catch (Exception e) {
                    TabHaobanSyncErrorLog tab = new TabHaobanSyncErrorLog();
                    tab.setDataId(dto.getUserid());
                    tab.setDataType(0);
                    tab.setLogType(0);
                    tab.setReason(e.getMessage());
                    tab.setTaskId(taskId);
                    tab.setWxEnterpriseId(wxEnterpriseId);
                    syncErrorLogService.add(tab);
                }
            }
        }
    }

    @Override
    public List<String> listUnBindClerk(String storeId) {
        List<String> storeIds = new ArrayList<String>();
        storeIds.add(storeId);
        List<String> clerkList = clerkService.getclerkListByStoreIds(storeIds);
        List<StaffClerkRelationDTO> staffRelationList = staffClerkRelationService.listByClerkIds(clerkList);
        List<String> clerkIds = staffRelationList.stream().filter(s -> s.getClerkId() != null).map(s -> s.getClerkId()).collect(Collectors.toList());
        List<String> unBindList = new ArrayList<>();
        for (String clerkId : clerkList) {
            if (!clerkIds.contains(clerkId)) {
                unBindList.add(clerkId);
            }
        }
        return unBindList;
    }

    @Override
    public String insertUserLoginLog(UserLoginLogDTO userLoginLogDTO) {
        return userLoginLogService.insertUserLoginLog(userLoginLogDTO);

    }

    @Override
    public int countByDepartmentIds(List<String> departmentIds) {
        if (departmentIds == null || departmentIds.isEmpty()) {
            return 0;
        }
        return staffDepartmentRelatedMapper.countByDepartmentIds(departmentIds);
    }

    @Override
    public List<StaffDTO> listByUserIdsAndWxEnterpriseId(List<String> userIds, String wxEnterpriseId) {
        List<StaffDTO> list = staffService.listByUserIdsAndWxEnterpriseId(userIds, wxEnterpriseId);
        if (list == null) {
            return Collections.EMPTY_LIST;
        }
        for (StaffDTO staffDTO : list) {
            List<String> departmentIds = new ArrayList<>();
            List<TabHaobanStaffDepartmentRelated> relatedList = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staffDTO.getStaffId());
            for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : relatedList) {
                departmentIds.add(tabHaobanStaffDepartmentRelated.getDepartmentId());
            }
            List<DepartmentDTO> departmentList = departmentService.listByIds(departmentIds);
            staffDTO.setDepartmentList(departmentList);
        }
        return list;
    }

    @Override
    public ServiceResponse saveSyncStaffByGic(GicClerkDTO clerkDTO) {
        ServiceResponse response = new ServiceResponse();
        if (clerkDTO == null) {
            response.setCode(0);
            response.setMessage("参数错误");
            return response;
        }

        if (StringUtils.isBlank(clerkDTO.getStoreId())) {
            logger.info("【店员同步】gicStoreId为空");
            response.setCode(0);
            response.setMessage("【店员同步】gicStoreId为空");
            return response;
        }
        TabHaobanDepartment department = departmentService.selectByRelatedId(clerkDTO.getStoreId());
        if (department == null) {
            logger.info("【店员同步】失败====department为空：{}", clerkDTO.getStoreId());
            response.setCode(0);
            response.setMessage("部门门店不存在");
            return response;
        }
        String departmentId = department.getDepartmentId();
        String wxEnterpriseId = department.getWxEnterpriseId();
        StaffDTO staff = geCcheckAndInitUser(wxEnterpriseId, clerkDTO.getNationcode(), clerkDTO.getPhoneNumber());
        if (staff == null) {
            if (clerkDTO.getStatus() != 1) {
                response.setCode(0);
                response.setMessage("成员当前状态非正常" + clerkDTO.getStatus());
                return response;
            }
            staff = new StaffDTO();
            staff.setWxEnterpriseId(wxEnterpriseId);
            staff.setClerkCode(clerkDTO.getClerkCode());
            staff.setActiveFlag(1);
            staff.setDepartmentIds(departmentId);
            staff.setHeadImg(clerkDTO.getHeadImgUrl());
            staff.setNationCode(clerkDTO.getNationcode());
            staff.setPhoneNumber(clerkDTO.getPhoneNumber());
            staff.setPostion(clerkDTO.getPostionName());
            staff.setSex(clerkDTO.getClerkGender());
            staff.setStaffName(clerkDTO.getClerkName());
            staff.setNickName(clerkDTO.getClerkName());
            staff.setSyncPostionFlag(1);
            StaffDepartmentRelatedDTO isCodeExist = this.getDepartmentIdAndCode(departmentId, clerkDTO.getClerkCode());
            logger.info("【gic导购新增】isCodeExist={}", JSON.toJSONString(isCodeExist));
            //如果这个code已经绑定了 先解绑
            if (isCodeExist != null) {
                isCodeExist.setClerkCode(null);
                staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, isCodeExist));
            }
            ServiceResponse hr = this.add(staff, department.getDepartmentId());
            if (hr.getCode() == 1) {
                StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, hr.getResult().toString());
                logger.info("【gic导购新增】staffRelated={}", JSON.toJSONString(staffRelated));
                //绑定
                if (staffRelated != null) {
                    staffRelated.setClerkCode(clerkDTO.getClerkCode());
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                }
            } else {
                response.setCode(0);
                response.setMessage(hr.getMessage());
                return response;
            }
        } else {
            String staffId = staff.getStaffId();
            logger.info("【员工修改】staff={},status={}", JSON.toJSONString(staff), clerkDTO.getStatus());
            //删除
            if (clerkDTO.getStatus() == 0) {
                StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, staffId);
                if (staffRelated != null) {
                    staffRelated.setUpdateTime(new Date());
                    staffRelated.setClerkCode(null);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                }
                //修改
            } else {
                staff.setHeadImg(clerkDTO.getHeadImgUrl());
                staff.setSex(clerkDTO.getClerkGender());
                //staff.setPostion(postion);
//	    		staff.setStaffName(staffName);
                if (staff.getActiveFlag() != null && staff.getActiveFlag() != 1) {
                    staff.setStaffName(clerkDTO.getClerkName());
                }
                List<StaffDepartmentRelatedDTO> relatedList = this.listStaffDepartmentByStaffId(staffId);
                Map<String, StaffDepartmentRelatedDTO> map = com.gic.commons.util.CollectionUtil.toMap(relatedList, "departmentId");
                String departmentIds = "";
                if (map.get(departmentId) == null) {
                    departmentIds = departmentId;
                }
                for (StaffDepartmentRelatedDTO staffDepartmentRelatedDTO : relatedList) {
                    departmentIds += "," + staffDepartmentRelatedDTO.getDepartmentId();
                }
                if (departmentIds.startsWith(",")) {
                    departmentIds = departmentIds.substring(1);
                }
                if (map.get(departmentId) == null) {
                    this.staffEdit(staff, departmentIds);
                }
                StaffDepartmentRelatedDTO staffRelated = this.getDepartmentIdAndStaffId(departmentId, staffId);
                StaffDepartmentRelatedDTO isCodeExist = this.getDepartmentIdAndCode(departmentId, clerkDTO.getClerkCode());
                //如果这个人已经绑定了  先解绑
                if (isCodeExist != null) {
                    staffRelated.setUpdateTime(new Date());
                    isCodeExist.setClerkCode(null);
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, isCodeExist));
                }
                //如果这个人是未绑定的 则绑定clerk
                if (staffRelated != null) {
                    staffRelated.setUpdateTime(new Date());
                    staffRelated.setClerkCode(clerkDTO.getClerkCode());
                    staffDepartmentRelatedMapper.updateByPrimaryKey(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, staffRelated));
                }
                this.staffEdit(staff, departmentIds);
            }
        }
        return response;
    }

    private String changeHeaderImageUrl(String headImgUrl) {

//		InputStream in = new URL(headImgUrl).openStream();
        try {
            byte[] data = ImageUtil.getImgbyte(headImgUrl);
//			byte[] data = IOUtils.toByteArray(in);
            PicUploadResDTO uploadPic = qqCloudPicService.uploadPic(GlobalVar.ctxPropertiesMap.get(GlobalInfo.QQPIC_KEY_ENTERPRISE), data);
            logger.info("腾讯云万象优图返回" + JSON.toJSONString(uploadPic));
            return uploadPic.downloadUrl;
        } catch (Exception e) {
            logger.info("上传腾讯云万象优图返回失败了：", e);
            return headImgUrl;
        }
    }

    /**
     * 删除企业微信
     *
     * @param staff
     * @return
     */
    private ServiceResponse delQywxStaff(TabHaobanStaff staff, WxEnterpriseDTO enterpriseDTO) {
        ServiceResponse ret = new ServiceResponse();
        DepartmentDTO tempFlagDepartment = departmentService.getTempFlagDepartment(enterpriseDTO.getWxEnterpriseId());
        if (null == tempFlagDepartment) {
            DepartmentDTO parentDepert = departmentService.getRootByEnterpriseId(enterpriseDTO.getWxEnterpriseId());
            tempFlagDepartment = new DepartmentDTO();
            tempFlagDepartment.setTempFlag(1);
            tempFlagDepartment.setDepartmentName("暂存部门");
            tempFlagDepartment.setWxEnterpriseId(enterpriseDTO.getWxEnterpriseId());
            tempFlagDepartment.setHasSonNode(false);
            tempFlagDepartment.setIsStore(0);
            tempFlagDepartment.setParentDepartmentId(parentDepert.getDepartmentId());
            ServiceResponse<DepartmentDTO> response = departmentApiService.add(tempFlagDepartment);
            if (response.getCode() != 1) {
                ret.setCode(0);
                ret.setMessage("暂存部门创建失败");
                return ret;
            }
            tempFlagDepartment = response.getResult();
        }
        try {
            staffEdit(EntityUtil.changeEntityByJSON(StaffDTO.class, staff), tempFlagDepartment.getDepartmentId());
        } catch (RuntimeException ex) {
            ret.setCode(0);
            ret.setMessage(ex.getMessage());
        } catch (Exception e) {
            ret.setCode(0);
            ret.setMessage("新增异常");
        } finally {
            return ret;
        }

    }

    @Override
    public ServiceResponse<String> getStaffQrcode(String clerkCode, String storeId) {
        ServiceResponse<String> resp = new ServiceResponse<>();
        List<String> clerkCodes = new ArrayList<>();
        clerkCodes.add(clerkCode);
        List<ClerkDTO> list = clerkService.listClerk(storeId, clerkCodes, true);
        if (CollectionUtils.isEmpty(list)) {
            logger.info("成员不存在：{}", clerkCode);
            return null;
        }
        ClerkDTO clerkDTO = list.get(0);

        StaffClerkRelationDTO relationDTO = staffClerkRelationService.getByClerkId(clerkDTO.getClerkId());
        if (relationDTO == null) {
            logger.info("成员没有关联:{}", clerkCode);
            return null;
        }
        String key = "qrcode-" + relationDTO.getStaffId();
        Object url = RedisUtil.getCache(key);
        if (null != url) {
            resp.setResult((String) url);
            return resp;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(relationDTO.getWxEnterpriseId());
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
            logger.info("没有配置secret:{}", wxEnterpriseDTO.getWxEnterpriseId());
            return null;
        }
        UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterpriseDTO.getCorpid(), secretSetting.getSecretVal(), relationDTO.getWxUserId());
        if (user != null) {
            RedisUtil.setCache(key, user.getQr_code(), 2L, TimeUnit.DAYS);
            resp.setResult(user.getQr_code());
            return resp;
        }
        return resp;
    }

    @Override
    public List<StaffDTO> listByWxUserId(String wxUserId) {
        List<TabHaobanStaff> list = staffMapper.listByWxUserId(wxUserId);
        return EntityUtil.changeEntityListByJSON(StaffDTO.class, list);
    }


    @Override
    public List<String> listBindClerkUserId(String storeId) {
        List<String> userIdList = new ArrayList<String>();
        List<String> storeIds = new ArrayList<String>();
        storeIds.add(storeId);
        List<String> clerkList = clerkService.getclerkListByStoreIds(storeIds);
        List<StaffClerkRelationDTO> staffRelationList = staffClerkRelationService.listByClerkIds(clerkList);
        if (CollectionUtils.isEmpty(staffRelationList)) {
            return Collections.EMPTY_LIST;
        }
        Map<String, StaffClerkRelationDTO> clerkIdMap = staffRelationList.stream().filter(s -> s.getClerkId() != null).collect(Collectors.toMap(StaffClerkRelationDTO::getClerkId, s -> s));
        Set<String> keySet = clerkIdMap.keySet();
        for (String clerkId : clerkList) {
            if (keySet.contains(clerkId)) {
                userIdList.add(clerkIdMap.get(clerkId).getWxUserId());
            }
        }
        return userIdList;
    }

    @Override
    public List<StaffClerkRelationDTO> listBindRelationUserId(String storeId) {
        List<StaffClerkRelationDTO> bindList = new ArrayList<StaffClerkRelationDTO>();

        List<String> storeIds = new ArrayList<String>();
        storeIds.add(storeId);
        List<String> clerkList = clerkService.getclerkListByStoreIds(storeIds);
        List<StaffClerkRelationDTO> staffRelationList = staffClerkRelationService.listByClerkIds(clerkList);
        Map<String, StaffClerkRelationDTO> clerkIdMap = staffRelationList.stream().filter(s -> s.getClerkId() != null).collect(Collectors.toMap(StaffClerkRelationDTO::getClerkId, s -> s));
        Set<String> keySet = clerkIdMap.keySet();
        for (String clerkId : clerkList) {
            if (keySet.contains(clerkId)) {
                bindList.add(clerkIdMap.get(clerkId));
            }
        }
        return bindList;
    }

    @Override
    public boolean activeStaff(String staffId) {
        TabHaobanStaff staff = new TabHaobanStaff();
        staff.setStaffId(staffId);
        staff.setActiveFlag(1);
        staffService.updateByPrimaryKey(staff);
        return true;
    }

    @Override
    public StaffDTO getWxStaffOne(String wxEnterpriseId) {
        TabHaobanStaff wxStaffByOne = staffService.getWxStaffByOne(wxEnterpriseId);
        return EntityUtil.changeEntityByJSON(StaffDTO.class, wxStaffByOne);
    }

    @Override
    public StaffDTO getAndAddSupperAdmin(String wxEnterpriseId) {
        TabHaobanStaff haobanStaff = staffMapper.selectSuperByWxEnterpriseId(wxEnterpriseId);
        if (haobanStaff != null) {
            return EntityUtil.changeEntityNew(StaffDTO.class, haobanStaff);
        }
        haobanStaff = new TabHaobanStaff();
        haobanStaff.setStaffName("超级管理员");
        haobanStaff.setWxEnterpriseId(wxEnterpriseId);
        haobanStaff.setSuperManagerFlag(1);
        String staffId = staffService.add(haobanStaff);
        haobanStaff.setStaffId(staffId);
        StaffDTO staffDTO = EntityUtil.changeEntityNew(StaffDTO.class, haobanStaff);
        return staffDTO;
    }

    @Override
    public StaffPrivacyUseLogDTO getStaffPrivacyByStaffId(String staffId) {
        return EntityUtil.changeEntity(StaffPrivacyUseLogDTO.class, staffPrivacyUseLogService.getByStaffId(staffId));
    }

    @Override
    public List<StaffDTO> listUserIdByWxEnterpriseId(String wxEnterpriseId) {
        return EntityUtil.changeEntityListNew(StaffDTO.class, staffMapper.listUserIdByWxEnterpriseId(wxEnterpriseId));
    }

    @Override
    public List<StaffDTO> listByOpenUserIdsAndWxEnterpriseId(List<String> openUserIds, String wxEnterpriseId) {
        List<StaffDTO> list = staffService.listByOpenUserIdsAndWxEnterpriseId(openUserIds, wxEnterpriseId);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.EMPTY_LIST;
        }
        for (StaffDTO staffDTO : list) {
            List<String> departmentIds = new ArrayList<>();
            List<TabHaobanStaffDepartmentRelated> relatedList = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staffDTO.getStaffId());
            for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : relatedList) {
                departmentIds.add(tabHaobanStaffDepartmentRelated.getDepartmentId());
            }
            List<DepartmentDTO> departmentList = departmentService.listByIds(departmentIds);
            staffDTO.setDepartmentList(departmentList);
        }
        return list;
    }

    @Override
    public int updateOpenUserIdsByUserId(String wxEnterpriseId, String wxUserId, String wxOpenUseId) {
        return staffService.updateOpenUserIdsByUserId(wxEnterpriseId, wxUserId, wxOpenUseId);
    }

    @Override
    public int updateOpenIdByStaffId(String staffId, String openId) {
        return staffMapper.updateOpenIdByStaffId(staffId, openId);
    }

    @Override
    public int updateQrCodeByStaffId(String staffId, String qrCode) {
        return staffMapper.updateQrCodeByStaffId(staffId, qrCode);
    }

    @Override
    public StaffQrCodeDTO getQrCodeByClerkId(String clerkId) {
        StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
        if (staffClerkRelationDTO == null) {
            return null;
        }
        String staffId = staffClerkRelationDTO.getStaffId();
        String wxEnterpriseId = staffClerkRelationDTO.getWxEnterpriseId();

        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
            logger.info("没有配置secret：wxEnterpriseId:{}", wxEnterpriseId);
            return null;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.info("企业不存在：wxEnterpriseId:{}", wxEnterpriseId);
            return null;
        }
        TabHaobanStaff staff = staffService.selectById(staffId);
        if (staff == null) {
            logger.info("员工不存在：staffId:{}", staffId);
            return null;
        }
        StaffQrCodeDTO staffQrCodeDTO = new StaffQrCodeDTO();
        staffQrCodeDTO.setStaffId(staffId);
        staffQrCodeDTO.setHeadImg(staff.getHeadImg());
        staffQrCodeDTO.setStaffName(staff.getStaffName());
        staffQrCodeDTO.setNickName(staff.getNickName());
        staffQrCodeDTO.setPhoneNumber(staff.getPhoneNumber());
        staffQrCodeDTO.setClerkCode(staffClerkRelationDTO.getClerkCode());

        String qrCode = staff.getQrCode();
        staffQrCodeDTO.setQrCode(qrCode);
        if (StringUtils.isNotBlank(qrCode)) {
            return staffQrCodeDTO;
        }

        //获取微信信息
        String wxUserId = staff.getWxUserId();
        UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterpriseDTO.getCorpid(), secretSetting.getSecretVal(), wxUserId);
        if (null == user) {
            logger.info("企业微信用户不存在:wxEnterpriseId:{}:wxUserId:{}", wxEnterpriseId, wxUserId);
            return staffQrCodeDTO;
        }
        String qr_code = user.getQr_code();
        staffMapper.updateQrCodeByStaffId(staffId, qr_code);
        staffQrCodeDTO.setQrCode(qr_code);
        return staffQrCodeDTO;
    }

}
